/********************************************************************************
** Form generated from reading UI file 'connectdlgbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONNECTDLGBASE_H
#define UI_CONNECTDLGBASE_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_CConnectDlgBase
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *lblList;
    QComboBox *cbxDirectory;
    QLabel *lblFilter;
    QLineEdit *edtFilter;
    QCheckBox *chbExpandAll;
    QTreeWidget *lvwServers;
    QHBoxLayout *hboxLayout;
    QLabel *lblServerAddr;
    QComboBox *cbxServerAddr;
    QToolButton *tbtDeleteServerAddr;
    QHBoxLayout *hboxLayout1;
    QSpacerItem *spacerItem;
    QPushButton *butCancel;
    QPushButton *butConnect;

    void setupUi(QDialog *CConnectDlgBase)
    {
        if (CConnectDlgBase->objectName().isEmpty())
            CConnectDlgBase->setObjectName("CConnectDlgBase");
        CConnectDlgBase->resize(648, 493);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/png/main/res/fronticon.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        CConnectDlgBase->setWindowIcon(icon);
        CConnectDlgBase->setSizeGripEnabled(true);
        vboxLayout = new QVBoxLayout(CConnectDlgBase);
        vboxLayout->setObjectName("vboxLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(-1, -1, -1, 0);
        lblList = new QLabel(CConnectDlgBase);
        lblList->setObjectName("lblList");

        horizontalLayout->addWidget(lblList);

        cbxDirectory = new QComboBox(CConnectDlgBase);
        cbxDirectory->setObjectName("cbxDirectory");

        horizontalLayout->addWidget(cbxDirectory);

        lblFilter = new QLabel(CConnectDlgBase);
        lblFilter->setObjectName("lblFilter");

        horizontalLayout->addWidget(lblFilter);

        edtFilter = new QLineEdit(CConnectDlgBase);
        edtFilter->setObjectName("edtFilter");

        horizontalLayout->addWidget(edtFilter);

        chbExpandAll = new QCheckBox(CConnectDlgBase);
        chbExpandAll->setObjectName("chbExpandAll");

        horizontalLayout->addWidget(chbExpandAll);


        vboxLayout->addLayout(horizontalLayout);

        lvwServers = new QTreeWidget(CConnectDlgBase);
        lvwServers->setObjectName("lvwServers");
        lvwServers->setEditTriggers(QAbstractItemView::NoEditTriggers);
        lvwServers->setTabKeyNavigation(true);

        vboxLayout->addWidget(lvwServers);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        lblServerAddr = new QLabel(CConnectDlgBase);
        lblServerAddr->setObjectName("lblServerAddr");
        lblServerAddr->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        hboxLayout->addWidget(lblServerAddr);

        cbxServerAddr = new QComboBox(CConnectDlgBase);
        cbxServerAddr->setObjectName("cbxServerAddr");
        cbxServerAddr->setEditable(true);

        hboxLayout->addWidget(cbxServerAddr);

        tbtDeleteServerAddr = new QToolButton(CConnectDlgBase);
        tbtDeleteServerAddr->setObjectName("tbtDeleteServerAddr");

        hboxLayout->addWidget(tbtDeleteServerAddr);


        vboxLayout->addLayout(hboxLayout);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName("hboxLayout1");
        spacerItem = new QSpacerItem(351, 25, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(spacerItem);

        butCancel = new QPushButton(CConnectDlgBase);
        butCancel->setObjectName("butCancel");

        hboxLayout1->addWidget(butCancel);

        butConnect = new QPushButton(CConnectDlgBase);
        butConnect->setObjectName("butConnect");

        hboxLayout1->addWidget(butConnect);


        vboxLayout->addLayout(hboxLayout1);


        retranslateUi(CConnectDlgBase);

        butConnect->setDefault(true);


        QMetaObject::connectSlotsByName(CConnectDlgBase);
    } // setupUi

    void retranslateUi(QDialog *CConnectDlgBase)
    {
        CConnectDlgBase->setWindowTitle(QCoreApplication::translate("CConnectDlgBase", "Connection Setup", nullptr));
        CConnectDlgBase->setStyleSheet(QString());
        lblList->setText(QCoreApplication::translate("CConnectDlgBase", "Directory", nullptr));
        lblFilter->setText(QCoreApplication::translate("CConnectDlgBase", "Filter", nullptr));
        chbExpandAll->setText(QCoreApplication::translate("CConnectDlgBase", "Show All Musicians", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = lvwServers->headerItem();
        ___qtreewidgetitem->setText(4, QCoreApplication::translate("CConnectDlgBase", "Version", nullptr));
        ___qtreewidgetitem->setText(3, QCoreApplication::translate("CConnectDlgBase", "Location", nullptr));
        ___qtreewidgetitem->setText(2, QCoreApplication::translate("CConnectDlgBase", "Musicians", nullptr));
        ___qtreewidgetitem->setText(1, QCoreApplication::translate("CConnectDlgBase", "Ping Time", nullptr));
        ___qtreewidgetitem->setText(0, QCoreApplication::translate("CConnectDlgBase", "Server Name", nullptr));
        lblServerAddr->setText(QCoreApplication::translate("CConnectDlgBase", "Server Address", nullptr));
        butCancel->setText(QCoreApplication::translate("CConnectDlgBase", "C&ancel", nullptr));
        butConnect->setText(QCoreApplication::translate("CConnectDlgBase", "&Connect", nullptr));
    } // retranslateUi

};

namespace Ui {
    class CConnectDlgBase: public Ui_CConnectDlgBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CONNECTDLGBASE_H
