/****************************************************************************
** Meta object code from reading C++ file 'soundbase.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/sound/soundbase.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'soundbase.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10CSoundBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto CSoundBase::qt_create_metaobjectdata<qt_meta_tag_ZN10CSoundBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CSoundBase",
        "ReinitRequest",
        "",
        "iSndCrdResetType",
        "ControllerInFaderLevel",
        "iChannelIdx",
        "iValue",
        "ControllerInPanValue",
        "ControllerInFaderIsSolo",
        "bIsSolo",
        "ControllerInFaderIsMute",
        "bIsMute",
        "ControllerInMuteMyself",
        "bMute",
        "MidiCCReceived",
        "ccNumber"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'ReinitRequest'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'ControllerInFaderLevel'
        QtMocHelpers::SignalData<void(int, int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { QMetaType::Int, 6 },
        }}),
        // Signal 'ControllerInPanValue'
        QtMocHelpers::SignalData<void(int, int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { QMetaType::Int, 6 },
        }}),
        // Signal 'ControllerInFaderIsSolo'
        QtMocHelpers::SignalData<void(int, bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { QMetaType::Bool, 9 },
        }}),
        // Signal 'ControllerInFaderIsMute'
        QtMocHelpers::SignalData<void(int, bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { QMetaType::Bool, 11 },
        }}),
        // Signal 'ControllerInMuteMyself'
        QtMocHelpers::SignalData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Signal 'MidiCCReceived'
        QtMocHelpers::SignalData<void(int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CSoundBase, qt_meta_tag_ZN10CSoundBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CSoundBase::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10CSoundBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10CSoundBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10CSoundBaseE_t>.metaTypes,
    nullptr
} };

void CSoundBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CSoundBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ReinitRequest((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->ControllerInFaderLevel((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->ControllerInPanValue((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 3: _t->ControllerInFaderIsSolo((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 4: _t->ControllerInFaderIsMute((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 5: _t->ControllerInMuteMyself((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->MidiCCReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CSoundBase::*)(int )>(_a, &CSoundBase::ReinitRequest, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CSoundBase::*)(int , int )>(_a, &CSoundBase::ControllerInFaderLevel, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (CSoundBase::*)(int , int )>(_a, &CSoundBase::ControllerInPanValue, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (CSoundBase::*)(int , bool )>(_a, &CSoundBase::ControllerInFaderIsSolo, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (CSoundBase::*)(int , bool )>(_a, &CSoundBase::ControllerInFaderIsMute, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (CSoundBase::*)(bool )>(_a, &CSoundBase::ControllerInMuteMyself, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (CSoundBase::*)(int )>(_a, &CSoundBase::MidiCCReceived, 6))
            return;
    }
}

const QMetaObject *CSoundBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CSoundBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10CSoundBaseE_t>.strings))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int CSoundBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void CSoundBase::ReinitRequest(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void CSoundBase::ControllerInFaderLevel(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void CSoundBase::ControllerInPanValue(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void CSoundBase::ControllerInFaderIsSolo(int _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void CSoundBase::ControllerInFaderIsMute(int _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void CSoundBase::ControllerInMuteMyself(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void CSoundBase::MidiCCReceived(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
