/****************************************************************************
** Meta object code from reading C++ file 'clientdlg.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/clientdlg.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'clientdlg.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10CClientDlgE_t {};
} // unnamed namespace

template <> constexpr inline auto CClientDlg::qt_create_metaobjectdata<qt_meta_tag_ZN10CClientDlgE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CClientDlg",
        "SendTabChange",
        "",
        "iTabIdx",
        "OnConnectDisconBut",
        "OnTimerSigMet",
        "OnTimerBuffersLED",
        "OnTimerCheckAudioDeviceOk",
        "OnTimerDetectFeedback",
        "OnTimerStatus",
        "OnTimerPing",
        "OnPingTimeResult",
        "iPingTime",
        "OnCLPingTimeWithNumClientsReceived",
        "CHostAddress",
        "InetAddr",
        "iNumClients",
        "OnControllerInFaderLevel",
        "iChannelIdx",
        "iValue",
        "OnControllerInPanValue",
        "OnControllerInFaderIsSolo",
        "bIsSolo",
        "OnControllerInFaderIsMute",
        "bIsMute",
        "OnControllerInMuteMyself",
        "bMute",
        "OnVersionAndOSReceived",
        "COSUtil::EOpSystemType",
        "strVersion",
        "OnCLVersionAndOSReceived",
        "OnLoadChannelSetup",
        "OnSaveChannelSetup",
        "OnOpenConnectionSetupDialog",
        "OnOpenUserProfileSettings",
        "OnOpenAudioNetSettings",
        "OnOpenAdvancedSettings",
        "OnOpenChatDialog",
        "OnOpenAnalyzerConsole",
        "OnOwnFaderFirst",
        "OnNoSortChannels",
        "OnSortChannelsByName",
        "OnSortChannelsByInstrument",
        "OnSortChannelsByGroupID",
        "OnSortChannelsByCity",
        "OnSortChannelsByChannel",
        "OnClearAllStoredSoloMuteSettings",
        "OnSetAllFadersToNewClientLevel",
        "OnAutoAdjustAllFaderLevels",
        "OnNumMixerPanelRowsChanged",
        "value",
        "OnSettingsStateChanged",
        "OnChatStateChanged",
        "OnLocalMuteStateChanged",
        "OnAudioReverbValueChanged",
        "OnReverbSelLClicked",
        "OnReverbSelRClicked",
        "OnFeedbackDetectionChanged",
        "state",
        "OnConClientListMesReceived",
        "CVector<CChannelInfo>",
        "vecChanInfo",
        "OnChatTextReceived",
        "strChatText",
        "OnLicenceRequired",
        "ELicenceType",
        "eLicenceType",
        "OnSoundDeviceChanged",
        "strError",
        "OnChangeChanGain",
        "iId",
        "fGain",
        "bIsMyOwnFader",
        "OnChangeChanPan",
        "fPan",
        "OnNewLocalInputText",
        "OnReqServerListQuery",
        "OnCreateCLServerListPingMes",
        "OnCreateCLServerListReqVerAndOSMes",
        "OnCreateCLServerListReqConnClientsListMes",
        "OnCLServerListReceived",
        "CVector<CServerInfo>",
        "vecServerInfo",
        "OnCLRedServerListReceived",
        "OnCLConnClientsListMesReceived",
        "OnClientIDReceived",
        "iChanID",
        "OnMuteStateHasChangedReceived",
        "bIsMuted",
        "OnCLChannelLevelListReceived",
        "CVector<uint16_t>",
        "vecLevelList",
        "OnConnectDlgAccepted",
        "OnDisconnected",
        "OnGUIDesignChanged",
        "OnMeterStyleChanged",
        "OnRecorderStateReceived",
        "ERecorderState",
        "eRecorderState",
        "SetMixerBoardDeco",
        "newRecorderState",
        "EGUIDesign",
        "eNewDesign",
        "OnAudioChannelsChanged",
        "OnNumClientsChanged",
        "iNewNumClients",
        "accept",
        "OnMIDIControllerUsageChanged",
        "bEnabled"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'SendTabChange'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'OnConnectDisconBut'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnTimerSigMet'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnTimerBuffersLED'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnTimerCheckAudioDeviceOk'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnTimerDetectFeedback'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnTimerStatus'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnTimerPing'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnPingTimeResult'
        QtMocHelpers::SlotData<void(int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Slot 'OnCLPingTimeWithNumClientsReceived'
        QtMocHelpers::SlotData<void(CHostAddress, int, int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 }, { QMetaType::Int, 12 }, { QMetaType::Int, 16 },
        }}),
        // Slot 'OnControllerInFaderLevel'
        QtMocHelpers::SlotData<void(const int, const int)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 18 }, { QMetaType::Int, 19 },
        }}),
        // Slot 'OnControllerInPanValue'
        QtMocHelpers::SlotData<void(const int, const int)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 18 }, { QMetaType::Int, 19 },
        }}),
        // Slot 'OnControllerInFaderIsSolo'
        QtMocHelpers::SlotData<void(const int, const bool)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 18 }, { QMetaType::Bool, 22 },
        }}),
        // Slot 'OnControllerInFaderIsMute'
        QtMocHelpers::SlotData<void(const int, const bool)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 18 }, { QMetaType::Bool, 24 },
        }}),
        // Slot 'OnControllerInMuteMyself'
        QtMocHelpers::SlotData<void(const bool)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 26 },
        }}),
        // Slot 'OnVersionAndOSReceived'
        QtMocHelpers::SlotData<void(COSUtil::EOpSystemType, QString)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 28, 2 }, { QMetaType::QString, 29 },
        }}),
        // Slot 'OnCLVersionAndOSReceived'
        QtMocHelpers::SlotData<void(CHostAddress, COSUtil::EOpSystemType, QString)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 2 }, { 0x80000000 | 28, 2 }, { QMetaType::QString, 29 },
        }}),
        // Slot 'OnLoadChannelSetup'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnSaveChannelSetup'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnOpenConnectionSetupDialog'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnOpenUserProfileSettings'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnOpenAudioNetSettings'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnOpenAdvancedSettings'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnOpenChatDialog'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnOpenAnalyzerConsole'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnOwnFaderFirst'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnNoSortChannels'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnSortChannelsByName'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnSortChannelsByInstrument'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnSortChannelsByGroupID'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnSortChannelsByCity'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnSortChannelsByChannel'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnClearAllStoredSoloMuteSettings'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnSetAllFadersToNewClientLevel'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnAutoAdjustAllFaderLevels'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnNumMixerPanelRowsChanged'
        QtMocHelpers::SlotData<void(int)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 50 },
        }}),
        // Slot 'OnSettingsStateChanged'
        QtMocHelpers::SlotData<void(int)>(51, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 50 },
        }}),
        // Slot 'OnChatStateChanged'
        QtMocHelpers::SlotData<void(int)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 50 },
        }}),
        // Slot 'OnLocalMuteStateChanged'
        QtMocHelpers::SlotData<void(int)>(53, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 50 },
        }}),
        // Slot 'OnAudioReverbValueChanged'
        QtMocHelpers::SlotData<void(int)>(54, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 50 },
        }}),
        // Slot 'OnReverbSelLClicked'
        QtMocHelpers::SlotData<void()>(55, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnReverbSelRClicked'
        QtMocHelpers::SlotData<void()>(56, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnFeedbackDetectionChanged'
        QtMocHelpers::SlotData<void(int)>(57, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 58 },
        }}),
        // Slot 'OnConClientListMesReceived'
        QtMocHelpers::SlotData<void(CVector<CChannelInfo>)>(59, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 60, 61 },
        }}),
        // Slot 'OnChatTextReceived'
        QtMocHelpers::SlotData<void(QString)>(62, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 63 },
        }}),
        // Slot 'OnLicenceRequired'
        QtMocHelpers::SlotData<void(ELicenceType)>(64, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 65, 66 },
        }}),
        // Slot 'OnSoundDeviceChanged'
        QtMocHelpers::SlotData<void(QString)>(67, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 68 },
        }}),
        // Slot 'OnChangeChanGain'
        QtMocHelpers::SlotData<void(int, float, bool)>(69, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 70 }, { QMetaType::Float, 71 }, { QMetaType::Bool, 72 },
        }}),
        // Slot 'OnChangeChanPan'
        QtMocHelpers::SlotData<void(int, float)>(73, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 70 }, { QMetaType::Float, 74 },
        }}),
        // Slot 'OnNewLocalInputText'
        QtMocHelpers::SlotData<void(QString)>(75, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 63 },
        }}),
        // Slot 'OnReqServerListQuery'
        QtMocHelpers::SlotData<void(CHostAddress)>(76, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'OnCreateCLServerListPingMes'
        QtMocHelpers::SlotData<void(CHostAddress)>(77, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'OnCreateCLServerListReqVerAndOSMes'
        QtMocHelpers::SlotData<void(CHostAddress)>(78, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'OnCreateCLServerListReqConnClientsListMes'
        QtMocHelpers::SlotData<void(CHostAddress)>(79, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'OnCLServerListReceived'
        QtMocHelpers::SlotData<void(CHostAddress, CVector<CServerInfo>)>(80, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 }, { 0x80000000 | 81, 82 },
        }}),
        // Slot 'OnCLRedServerListReceived'
        QtMocHelpers::SlotData<void(CHostAddress, CVector<CServerInfo>)>(83, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 }, { 0x80000000 | 81, 82 },
        }}),
        // Slot 'OnCLConnClientsListMesReceived'
        QtMocHelpers::SlotData<void(CHostAddress, CVector<CChannelInfo>)>(84, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 }, { 0x80000000 | 60, 61 },
        }}),
        // Slot 'OnClientIDReceived'
        QtMocHelpers::SlotData<void(int)>(85, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 86 },
        }}),
        // Slot 'OnMuteStateHasChangedReceived'
        QtMocHelpers::SlotData<void(int, bool)>(87, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 86 }, { QMetaType::Bool, 88 },
        }}),
        // Slot 'OnCLChannelLevelListReceived'
        QtMocHelpers::SlotData<void(CHostAddress, CVector<uint16_t>)>(89, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 2 }, { 0x80000000 | 90, 91 },
        }}),
        // Slot 'OnConnectDlgAccepted'
        QtMocHelpers::SlotData<void()>(92, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnDisconnected'
        QtMocHelpers::SlotData<void()>(93, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnGUIDesignChanged'
        QtMocHelpers::SlotData<void()>(94, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnMeterStyleChanged'
        QtMocHelpers::SlotData<void()>(95, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnRecorderStateReceived'
        QtMocHelpers::SlotData<void(ERecorderState)>(96, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 97, 98 },
        }}),
        // Slot 'SetMixerBoardDeco'
        QtMocHelpers::SlotData<void(const ERecorderState, const EGUIDesign)>(99, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 97, 100 }, { 0x80000000 | 101, 102 },
        }}),
        // Slot 'OnAudioChannelsChanged'
        QtMocHelpers::SlotData<void()>(103, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnNumClientsChanged'
        QtMocHelpers::SlotData<void(int)>(104, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 105 },
        }}),
        // Slot 'accept'
        QtMocHelpers::SlotData<void()>(106, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnMIDIControllerUsageChanged'
        QtMocHelpers::SlotData<void(bool)>(107, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 108 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CClientDlg, qt_meta_tag_ZN10CClientDlgE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CClientDlg::staticMetaObject = { {
    QMetaObject::SuperData::link<CBaseDlg::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10CClientDlgE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10CClientDlgE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10CClientDlgE_t>.metaTypes,
    nullptr
} };

void CClientDlg::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CClientDlg *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->SendTabChange((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->OnConnectDisconBut(); break;
        case 2: _t->OnTimerSigMet(); break;
        case 3: _t->OnTimerBuffersLED(); break;
        case 4: _t->OnTimerCheckAudioDeviceOk(); break;
        case 5: _t->OnTimerDetectFeedback(); break;
        case 6: _t->OnTimerStatus(); break;
        case 7: _t->OnTimerPing(); break;
        case 8: _t->OnPingTimeResult((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->OnCLPingTimeWithNumClientsReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 10: _t->OnControllerInFaderLevel((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 11: _t->OnControllerInPanValue((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 12: _t->OnControllerInFaderIsSolo((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 13: _t->OnControllerInFaderIsMute((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 14: _t->OnControllerInMuteMyself((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->OnVersionAndOSReceived((*reinterpret_cast<std::add_pointer_t<COSUtil::EOpSystemType>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 16: _t->OnCLVersionAndOSReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<COSUtil::EOpSystemType>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 17: _t->OnLoadChannelSetup(); break;
        case 18: _t->OnSaveChannelSetup(); break;
        case 19: _t->OnOpenConnectionSetupDialog(); break;
        case 20: _t->OnOpenUserProfileSettings(); break;
        case 21: _t->OnOpenAudioNetSettings(); break;
        case 22: _t->OnOpenAdvancedSettings(); break;
        case 23: _t->OnOpenChatDialog(); break;
        case 24: _t->OnOpenAnalyzerConsole(); break;
        case 25: _t->OnOwnFaderFirst(); break;
        case 26: _t->OnNoSortChannels(); break;
        case 27: _t->OnSortChannelsByName(); break;
        case 28: _t->OnSortChannelsByInstrument(); break;
        case 29: _t->OnSortChannelsByGroupID(); break;
        case 30: _t->OnSortChannelsByCity(); break;
        case 31: _t->OnSortChannelsByChannel(); break;
        case 32: _t->OnClearAllStoredSoloMuteSettings(); break;
        case 33: _t->OnSetAllFadersToNewClientLevel(); break;
        case 34: _t->OnAutoAdjustAllFaderLevels(); break;
        case 35: _t->OnNumMixerPanelRowsChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 36: _t->OnSettingsStateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 37: _t->OnChatStateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 38: _t->OnLocalMuteStateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 39: _t->OnAudioReverbValueChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 40: _t->OnReverbSelLClicked(); break;
        case 41: _t->OnReverbSelRClicked(); break;
        case 42: _t->OnFeedbackDetectionChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 43: _t->OnConClientListMesReceived((*reinterpret_cast<std::add_pointer_t<CVector<CChannelInfo>>>(_a[1]))); break;
        case 44: _t->OnChatTextReceived((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 45: _t->OnLicenceRequired((*reinterpret_cast<std::add_pointer_t<ELicenceType>>(_a[1]))); break;
        case 46: _t->OnSoundDeviceChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 47: _t->OnChangeChanGain((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 48: _t->OnChangeChanPan((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<float>>(_a[2]))); break;
        case 49: _t->OnNewLocalInputText((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 50: _t->OnReqServerListQuery((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1]))); break;
        case 51: _t->OnCreateCLServerListPingMes((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1]))); break;
        case 52: _t->OnCreateCLServerListReqVerAndOSMes((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1]))); break;
        case 53: _t->OnCreateCLServerListReqConnClientsListMes((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1]))); break;
        case 54: _t->OnCLServerListReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CVector<CServerInfo>>>(_a[2]))); break;
        case 55: _t->OnCLRedServerListReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CVector<CServerInfo>>>(_a[2]))); break;
        case 56: _t->OnCLConnClientsListMesReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CVector<CChannelInfo>>>(_a[2]))); break;
        case 57: _t->OnClientIDReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 58: _t->OnMuteStateHasChangedReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 59: _t->OnCLChannelLevelListReceived((*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CVector<uint16_t>>>(_a[2]))); break;
        case 60: _t->OnConnectDlgAccepted(); break;
        case 61: _t->OnDisconnected(); break;
        case 62: _t->OnGUIDesignChanged(); break;
        case 63: _t->OnMeterStyleChanged(); break;
        case 64: _t->OnRecorderStateReceived((*reinterpret_cast<std::add_pointer_t<ERecorderState>>(_a[1]))); break;
        case 65: _t->SetMixerBoardDeco((*reinterpret_cast<std::add_pointer_t<ERecorderState>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<EGUIDesign>>(_a[2]))); break;
        case 66: _t->OnAudioChannelsChanged(); break;
        case 67: _t->OnNumClientsChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 68: _t->accept(); break;
        case 69: _t->OnMIDIControllerUsageChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CClientDlg::*)(int )>(_a, &CClientDlg::SendTabChange, 0))
            return;
    }
}

const QMetaObject *CClientDlg::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CClientDlg::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10CClientDlgE_t>.strings))
        return static_cast<void*>(this);
    return CBaseDlg::qt_metacast(_clname);
}

int CClientDlg::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = CBaseDlg::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 70)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 70;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 70)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 70;
    }
    return _id;
}

// SIGNAL 0
void CClientDlg::SendTabChange(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
