/****************************************************************************
** Meta object code from reading C++ file 'jamcontroller.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/recorder/jamcontroller.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jamcontroller.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8recorder14CJamControllerE_t {};
} // unnamed namespace

template <> constexpr inline auto recorder::CJamController::qt_create_metaobjectdata<qt_meta_tag_ZN8recorder14CJamControllerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "recorder::CJamController",
        "RestartRecorder",
        "",
        "StopRecorder",
        "RecordingSessionStarted",
        "sessionDir",
        "EndRecorderThread",
        "Stopped",
        "ClientDisconnected",
        "iChID",
        "AudioFrame",
        "stChName",
        "CHostAddress",
        "RecHostAddr",
        "iNumAudChan",
        "CVector<int16_t>",
        "vecsData"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'RestartRecorder'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'StopRecorder'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'RecordingSessionStarted'
        QtMocHelpers::SignalData<void(QString)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'EndRecorderThread'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'Stopped'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ClientDisconnected'
        QtMocHelpers::SignalData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Signal 'AudioFrame'
        QtMocHelpers::SignalData<void(const int, const QString, const CHostAddress, const int, const CVector<int16_t>)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 }, { QMetaType::QString, 11 }, { 0x80000000 | 12, 13 }, { QMetaType::Int, 14 },
            { 0x80000000 | 15, 16 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CJamController, qt_meta_tag_ZN8recorder14CJamControllerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject recorder::CJamController::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8recorder14CJamControllerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8recorder14CJamControllerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8recorder14CJamControllerE_t>.metaTypes,
    nullptr
} };

void recorder::CJamController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CJamController *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->RestartRecorder(); break;
        case 1: _t->StopRecorder(); break;
        case 2: _t->RecordingSessionStarted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->EndRecorderThread(); break;
        case 4: _t->Stopped(); break;
        case 5: _t->ClientDisconnected((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->AudioFrame((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<CHostAddress>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<CVector<int16_t>>>(_a[5]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 4:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< CVector<int16_t> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CJamController::*)()>(_a, &CJamController::RestartRecorder, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CJamController::*)()>(_a, &CJamController::StopRecorder, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (CJamController::*)(QString )>(_a, &CJamController::RecordingSessionStarted, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (CJamController::*)()>(_a, &CJamController::EndRecorderThread, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (CJamController::*)()>(_a, &CJamController::Stopped, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (CJamController::*)(int )>(_a, &CJamController::ClientDisconnected, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (CJamController::*)(const int , const QString , const CHostAddress , const int , const CVector<int16_t> )>(_a, &CJamController::AudioFrame, 6))
            return;
    }
}

const QMetaObject *recorder::CJamController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *recorder::CJamController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8recorder14CJamControllerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int recorder::CJamController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void recorder::CJamController::RestartRecorder()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void recorder::CJamController::StopRecorder()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void recorder::CJamController::RecordingSessionStarted(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void recorder::CJamController::EndRecorderThread()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void recorder::CJamController::Stopped()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void recorder::CJamController::ClientDisconnected(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void recorder::CJamController::AudioFrame(const int _t1, const QString _t2, const CHostAddress _t3, const int _t4, const CVector<int16_t> _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2, _t3, _t4, _t5);
}
QT_WARNING_POP
