/********************************************************************************
** Form generated from reading UI file 'clientdlgbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CLIENTDLGBASE_H
#define UI_CLIENTDLGBASE_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include "audiomixerboard.h"
#include "levelmeter.h"
#include "multicolorled.h"

QT_BEGIN_NAMESPACE

class Ui_CClientDlgBase
{
public:
    QHBoxLayout *hboxLayout;
    QFrame *backgroundFrame;
    QVBoxLayout *verticalLayout_5;
    QHBoxLayout *horizontalLayout_3;
    QHBoxLayout *horizontalLayout_2;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalLayout_3;
    QLabel *pxlLogo;
    QSpacerItem *verticalSpacer;
    QVBoxLayout *vboxLayout;
    QVBoxLayout *vboxLayout1;
    QLabel *lblAudioReverb;
    QHBoxLayout *hboxLayout1;
    QSpacerItem *spacerItem;
    QSlider *sldAudioReverb;
    QSpacerItem *spacerItem1;
    QVBoxLayout *vboxLayout2;
    QRadioButton *rbtReverbSelL;
    QRadioButton *rbtReverbSelR;
    QSpacerItem *spacerItem2;
    QFrame *lineMeter;
    QVBoxLayout *vboxLayout3;
    QLabel *lblInputLEDMeter;
    QHBoxLayout *hboxLayout2;
    CLevelMeter *lbrInputLevelL;
    CLevelMeter *lbrInputLevelR;
    QHBoxLayout *hboxLayout3;
    QLabel *lblLevelMeterLeft;
    QLabel *lblLevelMeterRight;
    QFrame *lineUpperLowerLeft_2;
    QGridLayout *gridLayout;
    QLabel *lblBuffers;
    QLabel *lblDelay;
    QLabel *lblPing;
    QLabel *lblPingVal;
    QLabel *lblPingUnit;
    QLabel *lblDelayVal;
    QLabel *lblDelayUnit;
    CMultiColorLED *ledDelay;
    CMultiColorLED *ledBuffers;
    QFrame *lineUpperLowerLeft;
    QCheckBox *chbLocalMute;
    QCheckBox *chbSettings;
    QCheckBox *chbChat;
    QPushButton *butConnect;
    QFrame *linePanReverb;
    QVBoxLayout *verticalLayout;
    QLabel *lblGlobalInfoLabel;
    QVBoxLayout *verticalLayout_2;
    QLabel *lblConnectToServer;
    CAudioMixerBoard *MainMixerBoard;
    QLabel *lblUpdateCheck;

    void setupUi(QDialog *CClientDlgBase)
    {
        if (CClientDlgBase->objectName().isEmpty())
            CClientDlgBase->setObjectName("CClientDlgBase");
        CClientDlgBase->resize(511, 490);
        CClientDlgBase->setAcceptDrops(true);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/png/main/res/fronticon.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        CClientDlgBase->setWindowIcon(icon);
        CClientDlgBase->setSizeGripEnabled(true);
        hboxLayout = new QHBoxLayout(CClientDlgBase);
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 3, 3);
        backgroundFrame = new QFrame(CClientDlgBase);
        backgroundFrame->setObjectName("backgroundFrame");
        backgroundFrame->setMinimumSize(QSize(0, 0));
        backgroundFrame->setFrameShape(QFrame::NoFrame);
        backgroundFrame->setFrameShadow(QFrame::Plain);
        verticalLayout_5 = new QVBoxLayout(backgroundFrame);
        verticalLayout_5->setObjectName("verticalLayout_5");
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setSpacing(3);
        verticalLayout_4->setObjectName("verticalLayout_4");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName("verticalLayout_3");
        pxlLogo = new QLabel(backgroundFrame);
        pxlLogo->setObjectName("pxlLogo");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(pxlLogo->sizePolicy().hasHeightForWidth());
        pxlLogo->setSizePolicy(sizePolicy);
        pxlLogo->setPixmap(QPixmap(QString::fromUtf8(":/png/main/res/fronticon.png")));
        pxlLogo->setAlignment(Qt::AlignCenter);
        pxlLogo->setWordWrap(false);

        verticalLayout_3->addWidget(pxlLogo);

        verticalSpacer = new QSpacerItem(13, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);

        vboxLayout = new QVBoxLayout();
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setSpacing(3);
        vboxLayout1->setObjectName("vboxLayout1");
        lblAudioReverb = new QLabel(backgroundFrame);
        lblAudioReverb->setObjectName("lblAudioReverb");
        lblAudioReverb->setAlignment(Qt::AlignCenter);

        vboxLayout1->addWidget(lblAudioReverb);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName("hboxLayout1");
        spacerItem = new QSpacerItem(0, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(spacerItem);

        sldAudioReverb = new QSlider(backgroundFrame);
        sldAudioReverb->setObjectName("sldAudioReverb");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(sldAudioReverb->sizePolicy().hasHeightForWidth());
        sldAudioReverb->setSizePolicy(sizePolicy1);
        sldAudioReverb->setMinimumSize(QSize(0, 125));
        sldAudioReverb->setMaximumSize(QSize(16777215, 300));
        sldAudioReverb->setPageStep(1);
        sldAudioReverb->setOrientation(Qt::Vertical);
        sldAudioReverb->setTickPosition(QSlider::TicksBothSides);

        hboxLayout1->addWidget(sldAudioReverb);

        spacerItem1 = new QSpacerItem(0, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(spacerItem1);


        vboxLayout1->addLayout(hboxLayout1);


        vboxLayout->addLayout(vboxLayout1);

        vboxLayout2 = new QVBoxLayout();
        vboxLayout2->setSpacing(3);
        vboxLayout2->setObjectName("vboxLayout2");
        rbtReverbSelL = new QRadioButton(backgroundFrame);
        rbtReverbSelL->setObjectName("rbtReverbSelL");

        vboxLayout2->addWidget(rbtReverbSelL);

        rbtReverbSelR = new QRadioButton(backgroundFrame);
        rbtReverbSelR->setObjectName("rbtReverbSelR");

        vboxLayout2->addWidget(rbtReverbSelR);


        vboxLayout->addLayout(vboxLayout2);


        verticalLayout_3->addLayout(vboxLayout);

        spacerItem2 = new QSpacerItem(13, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(spacerItem2);


        horizontalLayout->addLayout(verticalLayout_3);

        lineMeter = new QFrame(backgroundFrame);
        lineMeter->setObjectName("lineMeter");
        lineMeter->setFrameShape(QFrame::Shape::VLine);
        lineMeter->setFrameShadow(QFrame::Shadow::Sunken);

        horizontalLayout->addWidget(lineMeter);

        vboxLayout3 = new QVBoxLayout();
        vboxLayout3->setObjectName("vboxLayout3");
        lblInputLEDMeter = new QLabel(backgroundFrame);
        lblInputLEDMeter->setObjectName("lblInputLEDMeter");
        lblInputLEDMeter->setAlignment(Qt::AlignCenter);

        vboxLayout3->addWidget(lblInputLEDMeter);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setObjectName("hboxLayout2");
        lbrInputLevelL = new CLevelMeter(backgroundFrame);
        lbrInputLevelL->setObjectName("lbrInputLevelL");
        sizePolicy1.setHeightForWidth(lbrInputLevelL->sizePolicy().hasHeightForWidth());
        lbrInputLevelL->setSizePolicy(sizePolicy1);
        lbrInputLevelL->setMinimumSize(QSize(19, 88));

        hboxLayout2->addWidget(lbrInputLevelL);

        lbrInputLevelR = new CLevelMeter(backgroundFrame);
        lbrInputLevelR->setObjectName("lbrInputLevelR");
        sizePolicy1.setHeightForWidth(lbrInputLevelR->sizePolicy().hasHeightForWidth());
        lbrInputLevelR->setSizePolicy(sizePolicy1);
        lbrInputLevelR->setMinimumSize(QSize(19, 88));

        hboxLayout2->addWidget(lbrInputLevelR);


        vboxLayout3->addLayout(hboxLayout2);

        hboxLayout3 = new QHBoxLayout();
        hboxLayout3->setObjectName("hboxLayout3");
        lblLevelMeterLeft = new QLabel(backgroundFrame);
        lblLevelMeterLeft->setObjectName("lblLevelMeterLeft");
        lblLevelMeterLeft->setAlignment(Qt::AlignCenter);

        hboxLayout3->addWidget(lblLevelMeterLeft);

        lblLevelMeterRight = new QLabel(backgroundFrame);
        lblLevelMeterRight->setObjectName("lblLevelMeterRight");
        lblLevelMeterRight->setAlignment(Qt::AlignCenter);

        hboxLayout3->addWidget(lblLevelMeterRight);


        vboxLayout3->addLayout(hboxLayout3);


        horizontalLayout->addLayout(vboxLayout3);


        verticalLayout_4->addLayout(horizontalLayout);

        lineUpperLowerLeft_2 = new QFrame(backgroundFrame);
        lineUpperLowerLeft_2->setObjectName("lineUpperLowerLeft_2");
        lineUpperLowerLeft_2->setFrameShape(QFrame::Shape::HLine);
        lineUpperLowerLeft_2->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_4->addWidget(lineUpperLowerLeft_2);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        lblBuffers = new QLabel(backgroundFrame);
        lblBuffers->setObjectName("lblBuffers");
        lblBuffers->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        lblBuffers->setWordWrap(false);

        gridLayout->addWidget(lblBuffers, 2, 0, 1, 1);

        lblDelay = new QLabel(backgroundFrame);
        lblDelay->setObjectName("lblDelay");
        lblDelay->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        lblDelay->setWordWrap(false);

        gridLayout->addWidget(lblDelay, 1, 0, 1, 1);

        lblPing = new QLabel(backgroundFrame);
        lblPing->setObjectName("lblPing");
        lblPing->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        lblPing->setWordWrap(false);

        gridLayout->addWidget(lblPing, 0, 0, 1, 1);

        lblPingVal = new QLabel(backgroundFrame);
        lblPingVal->setObjectName("lblPingVal");
        lblPingVal->setMinimumSize(QSize(34, 0));
        lblPingVal->setLayoutDirection(Qt::LeftToRight);
        lblPingVal->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblPingVal, 0, 1, 1, 1);

        lblPingUnit = new QLabel(backgroundFrame);
        lblPingUnit->setObjectName("lblPingUnit");
        lblPingUnit->setMinimumSize(QSize(22, 0));
        lblPingUnit->setMaximumSize(QSize(22, 16777215));

        gridLayout->addWidget(lblPingUnit, 0, 2, 1, 1);

        lblDelayVal = new QLabel(backgroundFrame);
        lblDelayVal->setObjectName("lblDelayVal");
        lblDelayVal->setMinimumSize(QSize(34, 0));
        lblDelayVal->setLayoutDirection(Qt::LeftToRight);
        lblDelayVal->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lblDelayVal, 1, 1, 1, 1);

        lblDelayUnit = new QLabel(backgroundFrame);
        lblDelayUnit->setObjectName("lblDelayUnit");
        lblDelayUnit->setMinimumSize(QSize(22, 0));
        lblDelayUnit->setMaximumSize(QSize(22, 16777215));

        gridLayout->addWidget(lblDelayUnit, 1, 2, 1, 1);

        ledDelay = new CMultiColorLED(backgroundFrame);
        ledDelay->setObjectName("ledDelay");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(ledDelay->sizePolicy().hasHeightForWidth());
        ledDelay->setSizePolicy(sizePolicy2);
        ledDelay->setMinimumSize(QSize(14, 14));
        ledDelay->setMaximumSize(QSize(14, 14));

        gridLayout->addWidget(ledDelay, 1, 3, 1, 1);

        ledBuffers = new CMultiColorLED(backgroundFrame);
        ledBuffers->setObjectName("ledBuffers");
        sizePolicy2.setHeightForWidth(ledBuffers->sizePolicy().hasHeightForWidth());
        ledBuffers->setSizePolicy(sizePolicy2);
        ledBuffers->setMinimumSize(QSize(14, 14));
        ledBuffers->setMaximumSize(QSize(14, 14));

        gridLayout->addWidget(ledBuffers, 2, 3, 1, 1);


        verticalLayout_4->addLayout(gridLayout);

        lineUpperLowerLeft = new QFrame(backgroundFrame);
        lineUpperLowerLeft->setObjectName("lineUpperLowerLeft");
        lineUpperLowerLeft->setFrameShape(QFrame::Shape::HLine);
        lineUpperLowerLeft->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_4->addWidget(lineUpperLowerLeft);

        chbLocalMute = new QCheckBox(backgroundFrame);
        chbLocalMute->setObjectName("chbLocalMute");

        verticalLayout_4->addWidget(chbLocalMute);

        chbSettings = new QCheckBox(backgroundFrame);
        chbSettings->setObjectName("chbSettings");

        verticalLayout_4->addWidget(chbSettings);

        chbChat = new QCheckBox(backgroundFrame);
        chbChat->setObjectName("chbChat");

        verticalLayout_4->addWidget(chbChat);

        butConnect = new QPushButton(backgroundFrame);
        butConnect->setObjectName("butConnect");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(butConnect->sizePolicy().hasHeightForWidth());
        butConnect->setSizePolicy(sizePolicy3);
        butConnect->setMinimumSize(QSize(120, 45));
        butConnect->setAutoDefault(false);

        verticalLayout_4->addWidget(butConnect);


        horizontalLayout_2->addLayout(verticalLayout_4);

        linePanReverb = new QFrame(backgroundFrame);
        linePanReverb->setObjectName("linePanReverb");
        linePanReverb->setFrameShape(QFrame::Shape::VLine);
        linePanReverb->setFrameShadow(QFrame::Shadow::Sunken);

        horizontalLayout_2->addWidget(linePanReverb);


        horizontalLayout_3->addLayout(horizontalLayout_2);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        lblGlobalInfoLabel = new QLabel(backgroundFrame);
        lblGlobalInfoLabel->setObjectName("lblGlobalInfoLabel");
        lblGlobalInfoLabel->setAlignment(Qt::AlignCenter);
        lblGlobalInfoLabel->setMargin(6);

        verticalLayout->addWidget(lblGlobalInfoLabel);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setSizeConstraint(QLayout::SetMinimumSize);
        verticalLayout_2->setContentsMargins(0, 9, -1, -1);
        lblConnectToServer = new QLabel(backgroundFrame);
        lblConnectToServer->setObjectName("lblConnectToServer");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(lblConnectToServer->sizePolicy().hasHeightForWidth());
        lblConnectToServer->setSizePolicy(sizePolicy4);
        lblConnectToServer->setAlignment(Qt::AlignCenter);

        verticalLayout_2->addWidget(lblConnectToServer);


        verticalLayout->addLayout(verticalLayout_2);

        MainMixerBoard = new CAudioMixerBoard(backgroundFrame);
        MainMixerBoard->setObjectName("MainMixerBoard");
        sizePolicy4.setHeightForWidth(MainMixerBoard->sizePolicy().hasHeightForWidth());
        MainMixerBoard->setSizePolicy(sizePolicy4);

        verticalLayout->addWidget(MainMixerBoard);

        lblUpdateCheck = new QLabel(backgroundFrame);
        lblUpdateCheck->setObjectName("lblUpdateCheck");

        verticalLayout->addWidget(lblUpdateCheck);


        horizontalLayout_3->addLayout(verticalLayout);


        verticalLayout_5->addLayout(horizontalLayout_3);


        hboxLayout->addWidget(backgroundFrame);

        QWidget::setTabOrder(butConnect, chbLocalMute);
        QWidget::setTabOrder(chbLocalMute, chbSettings);
        QWidget::setTabOrder(chbSettings, chbChat);
        QWidget::setTabOrder(chbChat, sldAudioReverb);
        QWidget::setTabOrder(sldAudioReverb, rbtReverbSelL);
        QWidget::setTabOrder(rbtReverbSelL, rbtReverbSelR);

        retranslateUi(CClientDlgBase);

        butConnect->setDefault(false);


        QMetaObject::connectSlotsByName(CClientDlgBase);
    } // setupUi

    void retranslateUi(QDialog *CClientDlgBase)
    {
        CClientDlgBase->setWindowTitle(QString());
        CClientDlgBase->setStyleSheet(QString());
        lblAudioReverb->setText(QCoreApplication::translate("CClientDlgBase", "Reverb", nullptr));
        rbtReverbSelL->setText(QCoreApplication::translate("CClientDlgBase", "Left", nullptr));
        rbtReverbSelR->setText(QCoreApplication::translate("CClientDlgBase", "Right", nullptr));
        lblInputLEDMeter->setText(QCoreApplication::translate("CClientDlgBase", "Input", nullptr));
        lblLevelMeterLeft->setText(QCoreApplication::translate("CClientDlgBase", "L", nullptr));
        lblLevelMeterRight->setText(QCoreApplication::translate("CClientDlgBase", "R", nullptr));
        lblBuffers->setText(QCoreApplication::translate("CClientDlgBase", "Jitter", nullptr));
        lblDelay->setText(QCoreApplication::translate("CClientDlgBase", "Delay", nullptr));
        lblPing->setText(QCoreApplication::translate("CClientDlgBase", "Ping", nullptr));
        lblPingVal->setText(QString());
        lblPingUnit->setText(QCoreApplication::translate("CClientDlgBase", "ms", nullptr));
        lblDelayVal->setText(QString());
        lblDelayUnit->setText(QCoreApplication::translate("CClientDlgBase", "ms", nullptr));
        chbLocalMute->setText(QCoreApplication::translate("CClientDlgBase", "&Mute Myself", nullptr));
        chbSettings->setText(QCoreApplication::translate("CClientDlgBase", "&Settings", nullptr));
        chbChat->setText(QCoreApplication::translate("CClientDlgBase", "&Chat", nullptr));
        butConnect->setText(QCoreApplication::translate("CClientDlgBase", "C&onnect", nullptr));
        lblGlobalInfoLabel->setText(QCoreApplication::translate("CClientDlgBase", "MUTED (Other people won't hear you)", nullptr));
        lblConnectToServer->setText(QCoreApplication::translate("CClientDlgBase", "Set up your audio, connect to a server and start jamming!", nullptr));
        lblUpdateCheck->setText(QCoreApplication::translate("CClientDlgBase", "Update check", nullptr));
    } // retranslateUi

};

namespace Ui {
    class CClientDlgBase: public Ui_CClientDlgBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CLIENTDLGBASE_H
