/****************************************************************************
** Meta object code from reading C++ file 'clientsettingsdlg.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/clientsettingsdlg.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'clientsettingsdlg.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18CClientSettingsDlgE_t {};
} // unnamed namespace

template <> constexpr inline auto CClientSettingsDlg::qt_create_metaobjectdata<qt_meta_tag_ZN18CClientSettingsDlgE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CClientSettingsDlg",
        "GUIDesignChanged",
        "",
        "MeterStyleChanged",
        "AudioAlertsChanged",
        "AudioChannelsChanged",
        "CustomDirectoriesChanged",
        "NumMixerPanelRowsChanged",
        "value",
        "MIDIControllerUsageChanged",
        "bEnabled",
        "OnTimerStatus",
        "OnNetBufValueChanged",
        "OnNetBufServerValueChanged",
        "OnAutoJitBufStateChanged",
        "OnEnableOPUS64StateChanged",
        "OnFeedbackDetectionChanged",
        "OnCustomDirectoriesChanged",
        "bDelete",
        "OnNewClientLevelEditingFinished",
        "OnInputBoostChanged",
        "OnSndCrdBufferDelayButtonGroupClicked",
        "QAbstractButton*",
        "button",
        "OnSoundcardActivated",
        "iSndDevIdx",
        "OnLInChanActivated",
        "iChanIdx",
        "OnRInChanActivated",
        "OnLOutChanActivated",
        "OnROutChanActivated",
        "OnAudioChannelsActivated",
        "OnAudioQualityActivated",
        "iQualityIdx",
        "OnGUIDesignActivated",
        "iDesignIdx",
        "OnMeterStyleActivated",
        "iMeterStyleIdx",
        "OnAudioAlertsChanged",
        "OnLanguageChanged",
        "strLanguage",
        "OnAliasTextChanged",
        "strNewName",
        "OnInstrumentActivated",
        "iCntryListItem",
        "OnCountryActivated",
        "OnCityTextChanged",
        "OnSkillActivated",
        "OnTabChanged",
        "OnMakeTabChange",
        "iTabIdx",
        "OnAudioPanValueChanged",
        "OnMidiDeviceActivated",
        "iMidiDevIdx",
        "OnLearnButtonClicked",
        "OnMidiCCReceived",
        "ccNumber",
        "OnMIDIPickupModeToggled",
        "checked"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'GUIDesignChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'MeterStyleChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'AudioAlertsChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'AudioChannelsChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'CustomDirectoriesChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'NumMixerPanelRowsChanged'
        QtMocHelpers::SignalData<void(int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Signal 'MIDIControllerUsageChanged'
        QtMocHelpers::SignalData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Slot 'OnTimerStatus'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnNetBufValueChanged'
        QtMocHelpers::SlotData<void(int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'OnNetBufServerValueChanged'
        QtMocHelpers::SlotData<void(int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'OnAutoJitBufStateChanged'
        QtMocHelpers::SlotData<void(int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'OnEnableOPUS64StateChanged'
        QtMocHelpers::SlotData<void(int)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'OnFeedbackDetectionChanged'
        QtMocHelpers::SlotData<void(int)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'OnCustomDirectoriesChanged'
        QtMocHelpers::SlotData<void(bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 18 },
        }}),
        // Slot 'OnNewClientLevelEditingFinished'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnInputBoostChanged'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnSndCrdBufferDelayButtonGroupClicked'
        QtMocHelpers::SlotData<void(QAbstractButton *)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'OnSoundcardActivated'
        QtMocHelpers::SlotData<void(int)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 25 },
        }}),
        // Slot 'OnLInChanActivated'
        QtMocHelpers::SlotData<void(int)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 27 },
        }}),
        // Slot 'OnRInChanActivated'
        QtMocHelpers::SlotData<void(int)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 27 },
        }}),
        // Slot 'OnLOutChanActivated'
        QtMocHelpers::SlotData<void(int)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 27 },
        }}),
        // Slot 'OnROutChanActivated'
        QtMocHelpers::SlotData<void(int)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 27 },
        }}),
        // Slot 'OnAudioChannelsActivated'
        QtMocHelpers::SlotData<void(int)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 27 },
        }}),
        // Slot 'OnAudioQualityActivated'
        QtMocHelpers::SlotData<void(int)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 33 },
        }}),
        // Slot 'OnGUIDesignActivated'
        QtMocHelpers::SlotData<void(int)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 35 },
        }}),
        // Slot 'OnMeterStyleActivated'
        QtMocHelpers::SlotData<void(int)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 37 },
        }}),
        // Slot 'OnAudioAlertsChanged'
        QtMocHelpers::SlotData<void(int)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'OnLanguageChanged'
        QtMocHelpers::SlotData<void(QString)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 40 },
        }}),
        // Slot 'OnAliasTextChanged'
        QtMocHelpers::SlotData<void(const QString &)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 42 },
        }}),
        // Slot 'OnInstrumentActivated'
        QtMocHelpers::SlotData<void(int)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 44 },
        }}),
        // Slot 'OnCountryActivated'
        QtMocHelpers::SlotData<void(int)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 44 },
        }}),
        // Slot 'OnCityTextChanged'
        QtMocHelpers::SlotData<void(const QString &)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 42 },
        }}),
        // Slot 'OnSkillActivated'
        QtMocHelpers::SlotData<void(int)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 44 },
        }}),
        // Slot 'OnTabChanged'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OnMakeTabChange'
        QtMocHelpers::SlotData<void(int)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 50 },
        }}),
        // Slot 'OnAudioPanValueChanged'
        QtMocHelpers::SlotData<void(int)>(51, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'OnMidiDeviceActivated'
        QtMocHelpers::SlotData<void(int)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 53 },
        }}),
        // Slot 'OnLearnButtonClicked'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'OnMidiCCReceived'
        QtMocHelpers::SlotData<void(int)>(55, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 56 },
        }}),
        // Slot 'OnMIDIPickupModeToggled'
        QtMocHelpers::SlotData<void(bool)>(57, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 58 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CClientSettingsDlg, qt_meta_tag_ZN18CClientSettingsDlgE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CClientSettingsDlg::staticMetaObject = { {
    QMetaObject::SuperData::link<CBaseDlg::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18CClientSettingsDlgE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18CClientSettingsDlgE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18CClientSettingsDlgE_t>.metaTypes,
    nullptr
} };

void CClientSettingsDlg::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CClientSettingsDlg *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->GUIDesignChanged(); break;
        case 1: _t->MeterStyleChanged(); break;
        case 2: _t->AudioAlertsChanged(); break;
        case 3: _t->AudioChannelsChanged(); break;
        case 4: _t->CustomDirectoriesChanged(); break;
        case 5: _t->NumMixerPanelRowsChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->MIDIControllerUsageChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->OnTimerStatus(); break;
        case 8: _t->OnNetBufValueChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->OnNetBufServerValueChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->OnAutoJitBufStateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->OnEnableOPUS64StateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->OnFeedbackDetectionChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 13: _t->OnCustomDirectoriesChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->OnNewClientLevelEditingFinished(); break;
        case 15: _t->OnInputBoostChanged(); break;
        case 16: _t->OnSndCrdBufferDelayButtonGroupClicked((*reinterpret_cast<std::add_pointer_t<QAbstractButton*>>(_a[1]))); break;
        case 17: _t->OnSoundcardActivated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 18: _t->OnLInChanActivated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 19: _t->OnRInChanActivated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 20: _t->OnLOutChanActivated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 21: _t->OnROutChanActivated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 22: _t->OnAudioChannelsActivated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 23: _t->OnAudioQualityActivated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 24: _t->OnGUIDesignActivated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 25: _t->OnMeterStyleActivated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 26: _t->OnAudioAlertsChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 27: _t->OnLanguageChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 28: _t->OnAliasTextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 29: _t->OnInstrumentActivated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 30: _t->OnCountryActivated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 31: _t->OnCityTextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 32: _t->OnSkillActivated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 33: _t->OnTabChanged(); break;
        case 34: _t->OnMakeTabChange((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 35: _t->OnAudioPanValueChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 36: _t->OnMidiDeviceActivated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 37: _t->OnLearnButtonClicked(); break;
        case 38: _t->OnMidiCCReceived((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 39: _t->OnMIDIPickupModeToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractButton* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CClientSettingsDlg::*)()>(_a, &CClientSettingsDlg::GUIDesignChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CClientSettingsDlg::*)()>(_a, &CClientSettingsDlg::MeterStyleChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (CClientSettingsDlg::*)()>(_a, &CClientSettingsDlg::AudioAlertsChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (CClientSettingsDlg::*)()>(_a, &CClientSettingsDlg::AudioChannelsChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (CClientSettingsDlg::*)()>(_a, &CClientSettingsDlg::CustomDirectoriesChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (CClientSettingsDlg::*)(int )>(_a, &CClientSettingsDlg::NumMixerPanelRowsChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (CClientSettingsDlg::*)(bool )>(_a, &CClientSettingsDlg::MIDIControllerUsageChanged, 6))
            return;
    }
}

const QMetaObject *CClientSettingsDlg::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CClientSettingsDlg::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18CClientSettingsDlgE_t>.strings))
        return static_cast<void*>(this);
    return CBaseDlg::qt_metacast(_clname);
}

int CClientSettingsDlg::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = CBaseDlg::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 40)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 40;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 40)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 40;
    }
    return _id;
}

// SIGNAL 0
void CClientSettingsDlg::GUIDesignChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void CClientSettingsDlg::MeterStyleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void CClientSettingsDlg::AudioAlertsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void CClientSettingsDlg::AudioChannelsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void CClientSettingsDlg::CustomDirectoriesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void CClientSettingsDlg::NumMixerPanelRowsChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void CClientSettingsDlg::MIDIControllerUsageChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
